@@ -9,7 +9,7 @@ use FileHandle;
 print "\n";
 
 check_tell_bug();
-my ($dbm_ext, $dbmopen_ext, $dbmopen_haslock)=dbm_test();
+my ($dbm_ext, $dbmopen_ext, $dbmopen_haslock)=guessoptions();
 print_dbm_module();
 check_db_file_pm();
 print_dbm_option($dbm_ext, $dbmopen_ext, $dbmopen_haslock);
@@ -39,7 +39,7 @@ sub check_tell_bug {
    return 0;
 }
 
-sub dbm_test {
+sub guessoptions {
    my (%DB, @filelist, @delfiles);
    my ($dbm_ext, $dbmopen_ext, $dbmopen_haslock);
 
@@ -144,7 +144,7 @@ sub check_savedsuid_support {
    $>=0;
    if ($>!=0) {
       print qq|Your system didn't have saved suid support,\n|.
-            qq|please set the following option in dbm.conf\n\n|.
+            qq|please set the following option in suid.conf\n\n|.
             qq|\tsavedsuid_support no\n\n\n|;
       return -1;
    }
@@ -160,35 +160,58 @@ use vars qw(%opentable);
 # it opens the file to get the handle if need,
 # than do lock operation on the related filehandle
 sub flock_lock {
-   my ($filename, $lockflag)=@_;
+   my ($filename, $lockflag, $perm)=@_;
+   ($filename =~ /^(.+)$/) && ($filename = $1);	# untaint ...
 
-   ($filename =~ /^(.+)$/) && ($filename = $1);	# untaint...
-   if ( (! -e $filename) && $lockflag ne LOCK_UN) {
-      sysopen(F, $filename, O_RDWR|O_CREAT, 0600) or return 0; # create file for lock
-      close(F);
-   }
+   my ($dev, $inode, $fh, $n, $retval);
 
-   my ($dev, $inode, $fh);
-   ($dev, $inode)=(stat($filename))[0,1];
-   return 0 if ($dev eq '' || $inode eq '');
+   # deal unlock first
+   if ($lockflag & LOCK_UN) {
+      return 1 if ( !-e $filename);
+      ($dev, $inode)=(stat($filename))[0,1];
+      return 0 if ($dev eq '' || $inode eq '');
 
-   if (defined($opentable{"$dev-$inode"}) ) {
-      $fh=$opentable{"$dev-$inode"};
-   } else { # handle not found, open it!
-      $fh=do { local *FH };
-      if (sysopen($fh, $filename, O_RDWR) ||	# try RDWR open first
-          sysopen($fh, $filename, O_RDONLY) ) {	# then RDONLY for readonly file
-         $opentable{"$dev-$inode"}=$fh;
+      if (defined($opentable{"$dev-$inode"}) ) {
+         $fh=$opentable{"$dev-$inode"}{fh};
+         $retval=flock($fh, LOCK_UN);
+         if ($retval) {
+            $opentable{"$dev-$inode"}{n}--;
+            if ($opentable{"$dev-$inode"}{n}==0) {
+               delete($opentable{"$dev-$inode"});
+               close($fh) if ( defined(fileno($fh)) );
+            }
+         }
       } else {
          return 0;
       }
+      return $retval;
+   }
+
+   # else are file lock
+   if (!-e $filename) {
+      $perm=0600 if (!$perm);
+      sysopen(F, $filename, O_RDWR|O_CREAT, $perm) or return 0; # create file for lock
+      close(F);
+   }
+   ($dev, $inode)=(stat($filename))[0,1];
+   return 0 if ($dev eq '' || $inode eq '');
+
+   if (!defined($opentable{"$dev-$inode"}) ) {
+      $fh=do { local *FH };
+      if (sysopen($fh, $filename, O_RDWR) ||	# try RDWR open first
+          sysopen($fh, $filename, O_RDONLY) ) {	# then RDONLY for readonly file
+         $opentable{"$dev-$inode"}{fh}=$fh;
+      } else {
+         return 0;
+      }
+   } else {
+      $fh=$opentable{"$dev-$inode"}{fh};
    }
 
    # turn nonblocking lock to  30 secs timeouted lock
    # so owm gets higher chance to success in case other ap locks same file for only few secs
    # turn blocking    lock to 120 secs timeouted lock
    # so openwebmaill won't hang because of file locking
-   my $retval;
    eval {
       local $SIG{ALRM} = sub { die "alarm\n" }; # NB: \n required
       if ( $lockflag & LOCK_NB ) {	# nonblocking lock
@@ -196,13 +219,11 @@ sub flock_lock {
       } else {
          alarm 120;
       }
-      $retval=flock($fh, $lockflag & (~LOCK_NB) );
+      $retval=flock($fh, $lockflag&(~LOCK_NB));
       alarm 0;
    };
-   if ($@) {	# eval error, it means timeout
-      $retval=0;
-   }
+   $retval=0 if ($@);	# eval error, it means timeout
+   $opentable{"$dev-$inode"}{n}++ if ($retval);
+
    return($retval);
 }
-
-
