@@ -3,6 +3,7 @@ use strict;
 
 use vars qw(%lasttimeofday);
 %lasttimeofday=();
+
 sub gettimeofday {
    my ($SYS_gettimeofday, $timeval, $timezone, $sec, $usec);
 
@@ -14,6 +15,10 @@ sub gettimeofday {
    return $sec +  $usec/1e6;
 }
 
+sub timeofday_init {
+   %lasttimeofday=();
+}
+
 sub timeofday_diff {
    my $tag=$_[0]||'default';
 
